"""
For Python services with an API service token
"""
import json
import requests
import time

from six.moves import urllib

import ovation.session as session

DEFAULT_AUTH_DOMAIN = 'https://ovation.auth0.com/'


def get_service_token(client_id, client_secret,
                      audience='https://api.ovation.io/',
                      auth=DEFAULT_AUTH_DOMAIN):
    """Creates a new session for a service account.

    Arguments
    ---------
    client_id : string
        Service account Client ID

    client_secret : string
        Service account Client Secret

    audience : string, optional
        API audience. Default: https://api.ovation.io/

    auth : string, optional
        Authorization domain. Default: https://ovation.auth0.com

    Returns
    -------
    token_info : Dict
        Session token info. Pass this to make_session.
    """

    token_url = urllib.parse.urljoin(auth, '/oauth/token')
    body = {'client_id': client_id,
            'client_secret': client_secret,
            'audience': audience,
            'grant_type': 'client_credentials'}

    response = requests.post(token_url,
                             json=body)

    response.raise_for_status()

    token = response.json()

    expires_at = token['expires_in'] + time.time()

    token['expires_at'] = expires_at

    return token


def make_session(token_info,
                 organization=0,
                 client_id=None,
                 client_secret=None,
                 auth=DEFAULT_AUTH_DOMAIN,
                 audience='https://api.ovation.io/',
                 api=session.DEFAULT_HOST):
    """

    :param token_info:
    :param organization:
    :param client_id:
    :param client_secret:
    :param auth:
    :param audience:
    :param api:
    :return: tuple((updated) token info, session)
    """
    if (token_info is None) or (time.time() > token_info['expires_at'] and
                                        (client_id is not None) and
                                        (client_secret is not None)):
        token_info = get_service_token(client_id=client_id,
                                       client_secret=client_secret,
                                       auth=auth,
                                       audience=audience)

    s = session.Session(token_info['access_token'],
                        org=organization,
                        api=api)

    return (token_info, s)



import uuid
import ovation.core as core
import ovation.session as session

from unittest.mock import Mock, sentinel, patch
from nose.tools import istest, assert_equal


@istest
def should_create_file():
    s = Mock(spec=session.Session)
    s.get = Mock()
    s.get.side_effect = [session.DataDict({'type': 'Project',
                                           '_id': make_uuid(),
                                           'links': session.DataDict({'self': sentinel.parent_self})})]
    s.post = Mock()
    file = {'entities': [{'type': 'File',
                          '_id': make_uuid()}]}
    s.post.return_value = file
    expected_name = 'file name'

    core.create_file(s, make_uuid(), expected_name)

    s.post.assert_called_once_with(sentinel.parent_self, data={'entities': [{'type': 'File',
                                                                             'attributes': {'name': expected_name}}]})

@istest
def should_create_folder():
    s = Mock(spec=session.Session)
    s.get = Mock()
    s.get.side_effect = [session.DataDict({'type': 'Project',
                                           '_id': make_uuid(),
                                           'links': session.DataDict({'self': sentinel.parent_self})})]
    s.post = Mock()
    folder = {'entities': [{'type': 'Folder',
            '_id': make_uuid()}]}

    s.post.return_value = folder
    expected_name = 'folder name'


    core.create_folder(s, make_uuid(), expected_name)

    s.post.assert_called_once_with(sentinel.parent_self, data={'entities': [{'type': 'Folder',
                                                                             'attributes': {'name': expected_name}}]})


def make_uuid():
    return str(uuid.uuid4())


@istest
def should_trash_entity_by_id():
    s = Mock(spec=session.Session)

    s.delete.return_value = sentinel.deleted
    s.path.return_value = sentinel.path

    id = 'entity-id'
    r = core.delete_entity(s, id)

    s.delete.assert_called_once_with(sentinel.path)

@istest
def should_trash_entity():
    s = Mock(spec=session.Session)

    s.delete.return_value = sentinel.deleted
    s.path.return_value = sentinel.path

    id = 'entity-id'
    r = core.delete_entity(s, {"_id": id})

    s.delete.assert_called_once_with(sentinel.path)


@istest
def should_undelete_entity():
    entity_id = make_uuid()

    s = Mock(spec=session.Session)
    s.get.return_value = session.DataDict({'_id': entity_id,
                                           'trash_info': sentinel.trash_info})
    s.put.return_value = session.DataDict({'_id': entity_id})
    s.path.return_value = "/api/v1/entities/{}".format(entity_id)

    core.undelete_entity(s, entity_id)

    s.get.assert_called_once_with(s.path.return_value, params={'trash': "true"})
    s.put.assert_called_once_with(s.path.return_value + "/restore", s.get.return_value)



@istest
def should_get_entity():
    s = Mock(spec=session.Session)
    s.get.return_value = sentinel.result
    s.path.return_value = sentinel.path

    core.get_entity(s, 'entity-id')

    s.get.assert_called_once_with(sentinel.path, params={'trash': "false"})

@istest
def should_return_existing_entity_dict():

    assert_equal(core.get_entity(sentinel.session, sentinel.entity_dict),
                 sentinel.entity_dict)


@istest
def should_add_link():
    entity = {'relationships': {'foo': {'self': 'self-url'}}}
    rel_url = 'self-url'

    s = Mock(spec=session.Session)
    s.post.return_value = sentinel.result

    expected_post = [{'target_id': sentinel.target,
                      'inverse_rel': sentinel.inverse_rel}]

    core.add_link(s, entity, target=sentinel.target, rel='foo', inverse_rel=sentinel.inverse_rel)

    s.post.assert_called_once_with(rel_url, data=expected_post)


@istest
def should_remove_link():
    entity = {'_id': sentinel.entity_id,
              'relationships': {'foo': {'self': 'self-url'}},
              'organization_id': sentinel.org}
    rel_url = 'self-url'

    s = Mock(spec=session.Session)
    s.get.return_value = [{'_id': sentinel.link_id,
                           'target_id': sentinel.target,
                           'inverse_rel': sentinel.inverse_rel},
                          {'target_id': sentinel.target2,
                           'inverse_rel': sentinel.inverse_rel2}]
    s.delete.return_value = sentinel.result
    s.path.return_value = sentinel.url

    core.remove_link(s, entity, target=sentinel.target, rel='foo')

    s.get.assert_called_once_with(rel_url)
    s.path.assert_called_with('relationships', entity_id=sentinel.link_id, org=sentinel.org)
    s.delete.assert_called_once_with(sentinel.url)



import ovation.lab.workflows as workflows

from ovation.session import DataDict,Session,simplify_response
from unittest.mock import Mock, sentinel, patch
from nose.tools import istest, assert_equal
from tqdm import tqdm


@istest
def should_create_activity():
    label = 'activity-label'
    workflow_id = 1
    workflow = {'relationships': {label: {'self': sentinel.activity_url}}}

    s = Mock(spec=Session)
    s.path.return_value = sentinel.workflow_path
    s.get.return_value = simplify_response({'workflow': workflow, 'resources': []})
    s.post.return_value = DataDict({'activity': sentinel.activity})

    activity = {}
    workflows.create_activity(s, workflow_id, label, activity=activity)

    s.path.assert_called_with('workflows', workflow_id)
    s.get.assert_called_with(sentinel.workflow_path)
    s.post.assert_called_with(sentinel.activity_url, data={'activity': activity})


@istest
@patch('ovation.lab.workflows.upload.upload_resource')
def should_create_activity_with_resources(upload):
    label = 'activity-label'
    workflow_id = 1
    workflow = {'relationships': {label: {'self': sentinel.activity_url}}}

    s = Mock(spec=Session)
    s.path.return_value = sentinel.workflow_path
    s.get.return_value = simplify_response({'workflow': workflow, 'resources': []})
    uuid = 'activity-uuid'
    s.post.return_value = DataDict({'activity': {'uuid': uuid}})

    activity = {}
    workflows.create_activity(s, workflow_id, label, activity=activity, resources={'foo': ['foo.txt']})

    s.path.assert_called_with('workflows', workflow_id)
    s.get.assert_called_with(sentinel.workflow_path)
    s.post.assert_called_with(sentinel.activity_url, data={'activity': {'complete': False}})
    upload.assert_called_with(s, uuid, 'foo.txt', label='foo', progress=tqdm)


@istest
@patch('ovation.lab.workflows.upload.upload_resource')
def should_add_custom_attributes_if_needed(upload):
    label = 'activity-label'
    workflow_id = 1
    workflow = {'relationships': {label: {'self': sentinel.activity_url}}}

    s = Mock(spec=Session)
    s.path.return_value = sentinel.workflow_path
    s.get.return_value = simplify_response({'workflow': workflow, 'resources': []})
    uuid = 'activity-uuid'
    s.post.return_value = DataDict({'activity': {'uuid': uuid}})

    activity = {
        'foo': 'bar'
    }
    workflows.create_activity(s, workflow_id, label, activity=activity, resources={'foo': ['foo.txt']})

    s.path.assert_called_with('workflows', workflow_id)
    s.get.assert_called_with(sentinel.workflow_path)
    s.post.assert_called_with(sentinel.activity_url, data={'activity': {'complete': False,
                                                                        'custom_attributes': activity}})
    upload.assert_called_with(s, uuid, 'foo.txt', label='foo', progress=tqdm)


@istest
@patch('ovation.lab.workflows.upload.upload_resource')
def should_handle_existing_custom_attributes(upload):
    label = 'activity-label'
    workflow_id = 1
    workflow = {'relationships': {label: {'self': sentinel.activity_url}}}

    s = Mock(spec=Session)
    s.path.return_value = sentinel.workflow_path
    s.get.return_value = simplify_response({'workflow': workflow, 'resources': []})
    uuid = 'activity-uuid'
    s.post.return_value = DataDict({'activity': {'uuid': uuid}})

    activity = {
        'foo': 'bar',
        'custom_attributes': {'bar': 'baz'}
    }
    workflows.create_activity(s, workflow_id, label, activity=activity, resources={'foo': ['foo.txt']})

    s.path.assert_called_with('workflows', workflow_id)
    s.get.assert_called_with(sentinel.workflow_path)
    s.post.assert_called_with(sentinel.activity_url, data={'activity': {'complete': False,
                                                                        'foo': 'bar',
                                                                        'custom_attributes': {'bar': 'baz'}}})
    upload.assert_called_with(s, uuid, 'foo.txt', label='foo', progress=tqdm)


@istest
def should_get_workflow_samples():
    workflow_id = 1
    s = Mock(spec=Session)
    s.path.return_value = sentinel.workflow_path
    s.get.return_value = {'workflow': {}, 'resources': [], 'samples': [{'type': 'Sample'}]}
    assert_equal(workflows.get_samples(s, workflow_id), [{'type': 'Sample'}])

@istest
def should_get_workflow():
    workflow_id = 1
    s = Mock(spec=Session)
    s.path.return_value = sentinel.workflow_path
    s.get.return_value = {'workflow': sentinel.workflow}

    assert_equal(workflows.get_workflow(s, workflow_id), sentinel.workflow)



import json
import logging
import os.path

import six

import ovation.core as core
import ovation.upload as upload
import ovation.session

from tqdm import tqdm


def _resolve_links(session, project, links=[], progress=tqdm):
    resolved_links = []
    for link in progress(links,
                         leave=False,
                         unit='links'):
        if isinstance(link, six.string_types) and os.path.isfile(link):
            logging.debug("Uploading input %s", link)
            revision = upload.upload_file(session, project,
                                          link)['revision']
            resolved_links.append(revision)
        else:
            resolved_links.append(core.get_entity(session, link))

    return resolved_links


def create_activity(session, project, name, inputs=[], outputs=[], related=[], progress=tqdm):
    """
    Creates a new Activity within the given project.

    Activity inputs, outputs and related files may be specified at creation. Inputs
    may be Sources Revisions. Outputs and Related files must be Revisions. Each
    may be specified as a UUID, entity dict, or local path. For local paths, the
    local file will be uploaded to the Activity's project.

    :param session: ovation.session.Session
    :param project: Project (dict or Id)
    :param name: activity name
    :param inputs: input Revisions or Sources (dicts or Ids), or local file paths
    :param outputs: output Revisions (dicts or Ids) or local file paths
    :param related: action Revisions (dicts or Ids) or local file paths
    :return: new activity
    """
    logging.debug("Getting project")
    project = core.get_entity(session, project)

    logging.debug("Resolving inputs")
    inputs = _resolve_links(session, project, inputs, progress=progress)

    logging.debug("Resolving outputs")
    outputs = _resolve_links(session, project, outputs, progress=progress)

    logging.debug("Resolving related")
    related = _resolve_links(session, project, related, progress=progress)

    activity = {'type': core.ACTIVITY_TYPE,
                'attributes': {'name': name},
                'relationships': {'parents': {'related': [project['_id']],
                                              'type': core.PROJECT_TYPE,
                                              'inverse_rel': 'activities',
                                              'create_as_inverse': True},
                                  'inputs': {'related': [input['_id'] for input in inputs],
                                             'type': core.REVISION_TYPE,
                                             'inverse_rel': 'activities'},
                                  'outputs': {'related': [output['_id'] for output in outputs],
                                             'type': core.REVISION_TYPE,
                                             'inverse_rel': 'origins'},
                                  'actions': {'related': [r['_id'] for r in related],
                                              'type': core.REVISION_TYPE,
                                              'inverse_rel': 'procedures'}}}

    result = session.post(session.path('activities'), data={'activities': [activity]})

    return ovation.session.simplify_response(result['activities'][0])


def add_inputs(session, activity, inputs=[], progress=tqdm):
    """
    Adds inputs to an existing activity.

    Inputs may be Sources or Revisions, specified as UUIDs or entity dics. In addition,
    local files may be specified by file path. Local files will be uploaded to the
    Activity's project and the newly created Revision added as an input to
    the Activity.


    :param session: ovation.session.Session
    :param activity: activity UUID or Dict
    :param inputs: array of UUIDs, dicts, or local file paths of inputs to add
    :return:
    """
    activity = core.get_entity(session, activity)
    project = _get_project(session, activity)
    for activity_input in progress(_resolve_links(session, project, links=inputs, progress=progress),
                                   unit='inputs'):
        activity_input = core.get_entity(session, activity_input)
        core.add_link(session, activity,
                      target=activity_input['_id'],
                      rel='inputs',
                      inverse_rel='activities')


def _get_project(session, activity):
    project = core.get_entity(session, activity['links']['_collaboration_roots'][0])
    return project


def remove_inputs(session, activity, inputs=[], progress=tqdm):
    """
    Removes inputs from the given activity.

    Inputs may be Sources or Revisions, specified as UUIDs or entity dics.

    :param session: ovation.session.Session
    :param activity: activity UUID or Dict
    :param inputs: array of UUIDs or dicts
    :return:
    """
    activity = core.get_entity(session, activity)
    for activity_input in progress(inputs, unit='inputs'):
        activity_input = core.get_entity(session, activity_input)
        core.remove_link(session, activity,
                         target=activity_input['_id'],
                         rel='inputs')


def add_outputs(session, activity, outputs=[], progress=tqdm):
    """
    Adds outputs to an existing activity.

    Outputs are Revisions, specified as UUIDs or entity dics. In addition,
    local files may be specified by file path. Local files will be uploaded to the
    Activity's project and the newly created Revision added as an output to
    the Activity.


    :param session: ovation.session.Session
    :param activity: activity UUID or Dict
    :param outputs: array of UUIDs, dicts, or local file paths of outputs to add
    """
    activity = core.get_entity(session, activity)
    project = _get_project(session, activity)
    for activity_output in progress(_resolve_links(session, project, links=outputs, progress=progress),
                                    unit='outputs'):
        activity_output = core.get_entity(session, activity_output)
        core.add_link(session, activity,
                      target=activity_output['_id'],
                      rel='outputs',
                      inverse_rel='origins')


def remove_outputs(session, activity, outputs=[], progress=tqdm):
    """
    Removes outputs from the given activity.

    Outputs are Revisions, specified as UUIDs or entity dics.

    :param session: ovation.session.Session
    :param activity: activity UUID or Dict
    :param outputs: array of UUIDs or dicts
    :return:
    """
    activity = core.get_entity(session, activity)
    for activity_output in progress(outputs,
                                    unit='outputs'):
        activity_output = core.get_entity(session, activity_output)
        core.remove_link(session, activity,
                         target=activity_output['_id'],
                         rel='outputs')


def add_related(session, activity, related=[], progress=tqdm):
    """
    Adds related Revisions to an existing activity.

    Related Revisions are specified as UUIDs or entity dics. In addition,
    local files may be specified by file path. Local files will be uploaded to the
    Activity's project and the newly created Revision added as a related Revision to
    the Activity.

    :param session: ovation.session.Session
    :param activity: activity UUID or Dict
    :param related: array of Revision UUIDs or Dicts
    :return:
    """
    activity = core.get_entity(session, activity)
    project = _get_project(session, activity)
    for activity_related in progress(_resolve_links(session, project, links=related, progress=progress),
                                     unit='files'):
        activity_related = core.get_entity(session, activity_related)
        core.add_link(session, activity,
                      target=activity_related['_id'],
                      rel='actions',
                      inverse_rel='procedures')


def remove_related(session, activity, related=[], progress=tqdm):
    """
    Removes related Revisions from the given activity.

    Related Revisions are Revisions, specified as UUIDs or entity dics.

    :param session: ovation.session.Session
    :param activity: activity UUID or Dict
    :param related: array of UUIDs or dicts
    :return:
    """
    activity = core.get_entity(session, activity)
    for activity_related in progress(related,
                                     unit='files'):
        activity_related = core.get_entity(session, activity_related)
        core.remove_link(session, activity,
                         target=activity_related['_id'],
                         rel='actions')


def create_main(args):
    session = args.session

    project_id = args.project
    if project_id is None:
        project_id = input('Project Id: ')

    name = args.name
    if name is None:
        name = input('Activity name: ')

    inputs = args.input or []
    outputs = args.output or []
    related = args.related or []

    activity = create_activity(session, project_id, name, inputs=inputs, outputs=outputs, related=related)

    if args.json:
        print(json.dumps({"activity": activity._id}))
        return

    if args.concise:
        print(activity._id)
        return

    print("Created activity: {}".format(activity._id))


def add_inputs_main(args):
    session = args.session
    activity = core.get_entity(session, args.activity_id)

    add_inputs(session, activity, args.inputs or [])

def remove_inputs_main(args):
    session = args.session
    activity = core.get_entity(session, args.activity_id)

    remove_inputs(session, activity, args.inputs or [])


def add_outputs_main(args):
    session = args.session
    activity = core.get_entity(session, args.activity_id)

    add_outputs(session, activity, args.outputs or [])

def remove_outputs_main(args):
    session = args.session
    activity = core.get_entity(session, args.activity_id)

    remove_outputs(session, activity, args.outputs or [])


def add_related_main(args):
    session = args.session
    activity = core.get_entity(session, args.activity_id)

    add_related(session, activity, args.related or [])

def remove_related_main(args):
    session = args.session
    activity = core.get_entity(session, args.activity_id)

    remove_related(session, activity, args.related or [])



import tqdm

import ovation.lab.upload as upload


def upload_incomplete_report(session, org=None, requisition_id=None, path=None, progress=tqdm.tqdm):
    if requisition_id is None:
        raise Exception("Requisition ID required")
    if path is None:
        raise Exception("Local report path required")
    if org is None:
        raise Exception("Organization required")

    req = session.get(session.path('requisition', requisition_id, include_org=False))

    resource = upload.upload_resource(session, req['requisition']['id'], path, progress=progress)

    return session.post(session.path('reports'), data={'report': {'organization_id': org,
                                                                  'requisition_id': requisition_id,
                                                                  'resource_id': resource.id}})



import os

import six
import argparse

import ovation.upload as upload
import ovation.download as download
import ovation.contents as contents
import ovation.session as session
import ovation.activities as activities


def main():
    parser = argparse.ArgumentParser(prog='python -m ovation.cli')
    parser.add_argument('-u', '--user', help='Ovation user email')
    parser.add_argument('-o', '--organization', help='Organization Id', default=0)
    parser.add_argument('-t', '--token', help='Ovation API token', default=None)

    subparsers = parser.add_subparsers(title='Available subcommands',
                                       description='Use `python -m ovation.cli <subcommand> -h` for additional help')

    # UPLOAD
    parser_upload = subparsers.add_parser('upload', description='Upload files to Ovation')
    parser_upload.add_argument('parent_id', help='Project or Folder UUID')
    parser_upload.add_argument('paths', nargs='+', help='Paths to local files or directories')
    parser_upload.set_defaults(func=upload.upload_paths)
    # upload.upload_paths(user=args.user,
    #                     parent_id=args.parent_id,
    #                     paths=args.paths)


    # DOWNLOAD
    parser_download = subparsers.add_parser('download', description='Download files from Ovation')
    parser_download.add_argument('entity_id', help='File or Revision UUID')
    parser_download.add_argument('-o', '--output', help='Output directory')
    parser_download.set_defaults(func=download.download_main)


    # CONTENTS
    parser_ls = subparsers.add_parser('list',
                                      aliases=['ls'],
                                      description='List Projects or Contents')

    parser_ls.add_argument('parent_id', nargs='?', help='(Optional) Project or Folder ID', default='')

    parser_ls.set_defaults(func=contents.list_contents_main)

    # ACTIVITIES
    activities_create = subparsers.add_parser('create-activity', description='Create a new Activity')
    activities_create.add_argument('-n', '--name', help='New activity name', required=True)
    activities_create.add_argument('-p', '--project', help='Project UUID')
    activities_create.add_argument('--json', action='store_true', help="Output Activity ID as JSON", default=False)
    activities_create.add_argument('--concise', help='Output Activity ID suitable for assignment to a shell variable', action='store_true', default=False)
    activities_create.add_argument('--input', action='append', help='Input Source or Revision UUIDs')
    activities_create.add_argument('--output', action='append', help='Outpur Revision UUIDs')
    activities_create.add_argument('--related', action='append', help='Related Revision UUIDs')
    activities_create.set_defaults(func=activities.create_main)

    activities_add_inputs = subparsers.add_parser('add-inputs', description='Add inputs to an activity')
    activities_add_inputs.add_argument('activity_id', help='Activity UUID')
    activities_add_inputs.add_argument('inputs', nargs='+', help='Input Source or Revision UUIDs')
    activities_add_inputs.set_defaults(func=activities.add_inputs_main)

    activities_remove_inputs = subparsers.add_parser('remove-inputs', description='Remove inputs to an activity')
    activities_remove_inputs.add_argument('activity_id', help='Activity UUID')
    activities_remove_inputs.add_argument('inputs', nargs='+', help='Input Source or Revision UUIDs')
    activities_remove_inputs.set_defaults(func=activities.remove_inputs_main)

    activities_add_outputs = subparsers.add_parser('add-outputs', description='Add outputs from an activity')
    activities_add_outputs.add_argument('activity_id', help='Activity UUID')
    activities_add_outputs.add_argument('outputs', nargs='+', help='Output Revision UUIDs')
    activities_add_outputs.set_defaults(func=activities.add_outputs_main)

    activities_remove_outputs = subparsers.add_parser('remove-outputs', description='Remove outputs from an activity')
    activities_remove_outputs.add_argument('activity_id', help='Activity UUID')
    activities_remove_outputs.add_argument('outputs', nargs='+', help='Output Revision UUIDs')
    activities_remove_outputs.set_defaults(func=activities.remove_outputs_main)

    activities_add_related = subparsers.add_parser('add-related', description='Add files related to an activity')
    activities_add_related.add_argument('activity_id', help='Activity UUID')
    activities_add_related.add_argument('related', nargs='+', help='Related Revision UUIDs')
    activities_add_related.set_defaults(func=activities.add_related_main)

    activities_remove_related = subparsers.add_parser('remove-related', description='Remove files related to an activity')
    activities_remove_related.add_argument('activity_id', help='Activity UUID')
    activities_remove_related.add_argument('related', nargs='+', help='Related Revision UUIDs')
    activities_remove_related.set_defaults(func=activities.remove_related_main)


    args = parser.parse_args()

    if args.user is None and args.token is None:
        args.user = input('Email: ')

    s = session.connect(args.user, token=args.token, org=args.organization)
    args.session = s

    args.func(args)


if __name__ == '__main__':
    main()



import mimetypes
import threading
import boto3
import six
import os
import math

import ovation.core as core

from boto3.s3.transfer import TransferConfig
from tqdm import tqdm


class ProgressCallback(object):
    def __init__(self, filename, progress=tqdm):
        self._filename = filename
        self._lock = threading.Lock()
        self._size = float(os.path.getsize(filename))
        self._progress = progress(unit='B',
                                  unit_scale=True,
                                  total=self._size,
                                  desc=os.path.basename(filename))

    def __call__(self, bytes_amount):
        # To simplify we'll assume this is hooked up
        # to a single filename.
        with self._lock:
            self._progress.update(bytes_amount)
            if bytes_amount >= self._size:
                self._progress.close()


def upload_folder(session, parent, directory_path, progress=tqdm):
    """
    Recursively uploads a folder to Ovation

    :param session: Session
    :param parent: Project or Folder root
    :param directory_path: local path to directory
    :param progress: if not None, wrap in a progress (i.e. tqdm). Default: tqdm
    """

    root_folder = parent
    for root, dirs, files in os.walk(directory_path):
        root_name = os.path.basename(root)
        if len(root_name) == 0:
            root_name = os.path.basename(os.path.dirname(root))

        root_folder = core.create_folder(session, root_folder, root_name)

        for f in files:
            path = os.path.join(root, f)
            file = core.create_file(session, root_folder, f)

            upload_revision(session, file, path, progress=progress)


def upload_file(session,
                parent,
                local_path_or_file_obj,
                name=None,
                content_type=None,
                progress=tqdm):
    """
    Upload a file to Ovation

    :param session: Session
    :param parent: Project or Folder root
    :param local_path_or_file_obj: local path to file
    :param progress: if not None, wrap in a progress (i.e. tqdm). Default: tqdm
    :param content_type: revision content type (default: infer from file name)
    :param name: file name (default: local path basename)
    :return: created File entity dictionary
    """

    if name is None:
        name = os.path.basename(local_path_or_file_obj)

    file = core.create_file(session, parent, name)
    return upload_revision(session,
                           file,
                           local_path_or_file_obj,
                           content_type=content_type,
                           file_name=name,
                           progress=progress)


def guess_content_type(file_name):
    content_type = mimetypes.guess_type(file_name)[0]
    if content_type is None:
        content_type = 'application/octet-stream'

    return content_type


MAX_PARTS = 10000
MB = boto3.s3.transfer.MB
GB = MB * 1024


def multipart_chunksize(n_bytes):
    """
    Calculates the correct chunk size for a file of n_bytes
    :param n_bytes: file size (bytes)
    :return: multipart chunk size (bytes)
    """

    chunk_size = math.ceil(n_bytes / MAX_PARTS)

    return max(chunk_size, 8 * MB)


def upload_revision(session,
                    parent_file,
                    local_path,
                    progress=tqdm,
                    chunk_size=multipart_chunksize,
                    content_type=None,
                    file_name=None):
    """
    Upload a new `Revision` to `parent_file`. File is uploaded from `local_path` to
    the Ovation cloud, and the newly created `Revision` version is set.
    :param session: ovation.connection.Session
    :param parent_file: file entity dictionary or file ID string
    :param local_path: local path
    :param progress: if not None, wrap in a progress (i.e. tqdm). Default: tqdm
    :param content_type: revision content type (default: infer from file name)
    :param file_name: revision file name (default: infer from local path)
    :return: new `Revision` entity dicitonary
    """

    if isinstance(parent_file, six.string_types):
        parent_file = session.get(session.path('file', entity_id=parent_file))

    if file_name is None:
        file_name = os.path.basename(local_path)

    if content_type is None:
        content_type = guess_content_type(file_name)

    r = session.post(parent_file['links']['self'],
                     data={'entities': [{'type': 'Revision',
                                         'attributes': {'name': file_name,
                                                        'content_type': content_type}}]})
    revision = r['entities'][0]
    aws = r['aws'][0]['aws']  # Returns an :aws for each created Revision

    try:
        upload_to_aws(aws, content_type, local_path, progress, chunk_size=chunk_size)
        return session.put(revision['links']['upload-complete'], entity=None)
    except Exception as err:
        return session.put(revision['links']['upload-failed'], entity=None)


def upload_to_aws(aws,
                  content_type,
                  local_path_or_fileobj,
                  progress,
                  chunk_size=multipart_chunksize):
    aws_session = boto3.Session(aws_access_key_id=aws['access_key_id'],
                                aws_secret_access_key=aws['secret_access_key'],
                                aws_session_token=aws['session_token'])
    s3 = aws_session.resource('s3')
    file_obj = s3.Object(aws['bucket'], aws['key'])
    args = {'ContentType': content_type,
            'ServerSideEncryption': 'AES256'}

    if isinstance(local_path_or_fileobj, six.string_types):
        transfer_config = TransferConfig(multipart_chunksize=chunk_size(os.path.getsize(local_path_or_fileobj)))

        if progress:
            file_obj.upload_file(local_path_or_fileobj,
                                 ExtraArgs=args,
                                 Callback=ProgressCallback(local_path_or_fileobj, progress=progress),
                                 Config=transfer_config)
        else:
            file_obj.upload_file(local_path_or_fileobj,
                                 ExtraArgs=args,
                                 Config=transfer_config)
    else:
        if progress:
            file_obj.upload_fileobj(local_path_or_fileobj,
                                    ExtraArgs=args,
                                    Callback=ProgressCallback(local_path_or_fileobj, progress=progress))
        else:
            file_obj.upload_fileobj(local_path_or_fileobj,
                                    ExtraArgs=args)


def upload_paths(args):
    session = args.session
    parent_id = args.parent_id
    paths = args.paths
    if paths is None:
        return

    for p in paths:
        print('Uploading {}'.format(p))
        if os.path.isdir(p):
            upload_folder(session, parent_id, p)
        else:
            upload_file(session, parent_id, p)



import six
import os

import ovation.upload as upload

from tqdm import tqdm


def _create_resource_group(session, activity, name, parent=None):
    body = {"resource_group": {
        "name": name,
        "activity_id": activity
    }}

    if parent is not None:
        body['resource_group']['parent_id'] = parent

    return session.post(session.path('resource_groups'), data=body)


def upload_resource(session, entity_id, local_path, label=None, resource_group=None, progress=tqdm):
    """
    Upload a single local file as a Resource

    :param session: Session
    :param entity_id: owning entity UUID
    :param local_path: local file path
    :param label: Resource label
    :param resource_group: (optional) ResourceGroup dict
    :param progress: (optional) progress monitor
    :return: new Resource dict
    """

    file_name = os.path.basename(local_path)
    content_type = upload.guess_content_type(file_name)

    data = {'resource': {'entity_id': entity_id,
                         'path': file_name}}
    if label:
        data['resource']['label'] = label

    if resource_group is not None:
        data['resource']['resource_group_id'] = resource_group.id

    r = session.post(session.path('resources'), data=data)
    aws = r['aws']

    upload.upload_to_aws(aws, content_type, local_path, progress)

    metadata = session.get(session.path('resources', r.id) + "/metadata")

    r.version = metadata.version_id
    r.type = 'resource'

    return session.put(session.path('resources', r.id), entity=r)


def upload_resource_group(session, activity, local_directory_path, label=None, progress=tqdm, parent_id=None):
    """
    Recursively uploads a folder to Ovation as a ResourceGroup

    :param session: Session
    :param local_directory_path: local path to directory
    :param activity: activity UUID or dict
    :param label: Resources' label
    :param progress: if not None, wrap in a progress (i.e. tqdm). Default: tqdm
    """

    if not isinstance(activity, six.string_types):
        activity = activity['uuid']

    root_name = os.path.basename(local_directory_path)
    if len(root_name) == 0:
        root_name = os.path.basename(os.path.dirname(local_directory_path))
    resource_group = _create_resource_group(session, activity, root_name, parent=parent_id)

    directory_iterator = os.scandir(local_directory_path)
    for entry in directory_iterator:
        if entry.is_file():
            upload_resource(session, activity, entry.path, label=label, resource_group=resource_group, progress=progress)

        elif entry.is_dir():
            upload_resource_group(session, activity, entry.path, label=label, progress=progress, parent_id=resource_group.id)

    return resource_group


def upload_resources(session, activity, resources, progress=tqdm):
    """
    Uploads a resources collection to an activity.

    :param session:  ovation.session.Session
    :param activity: activity UUID string or dict
    :param resources: local path(s) to activity Resources (by label)
    """

    if not isinstance(activity, six.string_types):
        activity = activity['uuid']

    for (label, paths) in six.iteritems(resources):
        for local_path in paths:
            upload_resource(session, activity, local_path, label=label, progress=progress)



import texttable
import logging

import ovation.core as core

create_file = core.create_file
create_folder = core.create_folder

def walk(session, parent, recurse=False):
    """
    Walks through the directory from the specified parent, yielding a 5-tuple of 'parent', 'folders',
    'files'.

    If recurse is set to true, then this function will continue through all sub-folders, otherwise
    it will yield only the contents of the specified parent.

    You can get the breadcrumb path of parent with `get_breadcrumbs`. You can get the HEAD revisions of file(s) with
    `get_head_revisions`. This example shows how to calculate the total byte size of all head revisions in a directory
    tree::

    total_bytes = 0
    for (parent, folders, files) in contents.walk(session, parent):
        for f in files:
            rev = contents.get_head_revision(session, f)
            total_bytes += rev.attributes.get('content_length', 0)

    print("Total bytes: {}".format(total_bytes))

    :param session: ovation.session.Session
    :param parent: Project or Folder dict or ID
    :returns generator of 5-tuple: `(parent, folders, files)`
    """

    folders = []
    files = []

    # if specified parent param is only a string id, will get entity object,
    # otherwise will use passed in parent param
    parent = core.get_entity(session, parent)

    entries = get_contents(session, parent)

    for file in entries['files']:
        files.append(file)

    for folder in entries['folders']:
        folders.append(folder)

    # yield
    yield parent, folders, files

    # recurse into sub-directories
    if recurse:
        for folder in folders:
            yield from walk(session, folder, recurse)


def get_contents(session, parent):
    """
    Gets all files and folders of parent
    :param session: ovation.session.Session
    :param parent: Project or Folder dict or ID
    :return: Dict of 'files' and 'folders'
    """

    p = core.get_entity(session, parent)

    if p is None:
        return None

    return {'files': session.get(p.relationships.files.related),
            'folders': session.get(p.relationships.folders.related)}


def get_head_revision(session, file):
    """
    Retrieves the head revision of file specified
    :param session: ovation.session.Session
    :param file: File dict or ID
    :return: Revision dict or None if there are no head revisions
    """

    file = core.get_entity(session, file)

    headRevisions = session.get(file.links.heads)
    if (headRevisions):
        return headRevisions[0]
    else:
        logging.warning("No head revisions found for file " + file.attributes.name)
        return None


def get_breadcrumbs(session, entity):
    """
    Returns the directory path of the entity (folder or file specified)
    :param session: ovation.session.Session
    :param entity: File / Folder dict or ID
    :return: string representing directory path of specified entity (e.g. "project1/folder1/file1.txt")
    """

    entity_directory_path = ""

    entity = core.get_entity(session, entity)

    if not entity.type in [core.FILE_TYPE, core.FOLDER_TYPE]:
        raise ValueError("entity is not a File or Folder")

    breadcrumb_list = session.get(session.path(resource="breadcrumbs"), params={"id": entity['_id']})

    if (breadcrumb_list):

        # service returns an array of multiple breadcrumbs
        # since parents (i.e folders or projects) can only have one breadcrumb, take the first and only one returned
        first_breadcrumb = breadcrumb_list[0]

        # to print out the path, reverse the list since it puts the project last
        first_breadcrumb.reverse()

        for crumb in first_breadcrumb:
            entity_directory_path += crumb['name']
            if (crumb['type'] == "Folder" or crumb['type'] == "Project"):
                entity_directory_path += "/"

    return entity_directory_path


def _get_head(session, file):
    return {'file': file._id,
            'revision': session.get(file.links.heads)[0]._id}


def list_contents_main(args):
    session = args.session
    parent_id = args.parent_id

    if parent_id is None or parent_id == '':

        table = texttable.Texttable(max_width=0)
        # table.set_deco(texttable.Texttable.HEADER)
        table.set_cols_align(["l", "l"])
        table.add_rows([['Name', 'ID']])

        for p in core.get_projects(session):
            table.add_row([p.attributes.name, p._id])

        print(table.draw())

    else:
        contents = get_contents(session, parent_id)
        files = contents['files']
        folders = contents['folders']

        table = texttable.Texttable(max_width=0)
        table.set_deco(texttable.Texttable.HEADER)
        table.set_cols_align(['l', 'l', 'l'])
        # table.set_cols_width([])
        table.add_rows([['Name', 'Modified', 'ID']])
        for e in sorted(files + folders, key=lambda e: e.attributes.name):
            if e.type == core.FOLDER_TYPE:
                name = e.attributes.name + "/"
                # head = ''
            else:
                name = e.attributes.name
                # head = revisions[e._id]

            table.add_row([name, e.attributes['updated-at'], e._id])

        print(table.draw())



from unittest.mock import patch, Mock, sentinel

from ovation.session import Session, DataDict
import ovation.lab.reports as reports


@patch('ovation.lab.upload.upload_resource')
def test_uploads_incomplete_report_resource(upload_resource):
    s = Mock(spec=Session)
    req_id = sentinel.req_id
    path = sentinel.path

    s.get.return_value = DataDict({'requisition': {'id': req_id}})

    reports.upload_incomplete_report(s,
                                     org=sentinel.org_id,
                                     requisition_id=req_id,
                                     path=path,
                                     progress=None)

    upload_resource.assert_called_with(s, req_id, path, progress=None)


@patch('ovation.lab.upload.upload_resource')
def test_creates_incomplete_report(upload_resource):
    s = Mock(spec=Session)
    req_id = sentinel.req_id
    path = sentinel.path

    s.get.return_value = DataDict({'requisition': {'id': req_id}})
    s.path.return_value = sentinel.reports_path

    upload_resource.return_value = DataDict({'id': sentinel.resource_id})

    reports.upload_incomplete_report(s,
                                     org=sentinel.org_id,
                                     requisition_id=req_id,
                                     path=path,
                                     progress=None)

    s.post.assert_called_with(sentinel.reports_path,
                              data={'report': {'organization_id': sentinel.org_id,
                                               'requisition_id': req_id,
                                               'resource_id': sentinel.resource_id}})



from unittest.mock import sentinel, Mock, patch
from nose.tools import istest, assert_is_not_none

import ovation.activities as activities
import ovation.session as session
import ovation.core as core


@istest
@patch('ovation.session.simplify_response')
@patch('ovation.upload.upload_file')
@patch('os.path.isfile')
def should_create_activity(isfile, upload_file, simplify_response):
    revision = {'type': 'Revision',
                '_id': sentinel.revision_id}
    source = {'type': 'Source',
              '_id': sentinel.source_id}

    proj = {'type': 'Project',
            '_id': sentinel.project_id,
            'links': {'self': sentinel.project_self}}

    output_revision = {'type': core.REVISION_TYPE,
                       '_id': sentinel.output_revision_id}

    s = Mock(spec=session.Session)
    post_activity_return = {'type': core.ACTIVITY_TYPE}
    s.post.return_value = {'activities': [post_activity_return]}
    simplify_response.return_value = sentinel.activity
    s.path.return_value = sentinel.activities_url

    output_path = '/some/path.txt'
    isfile.return_value = True
    upload_file.return_value = {'revision': output_revision}

    expected_post = {'type': core.ACTIVITY_TYPE,
                     'attributes': {'name': sentinel.activity_name},
                     'relationships': {'parents': {'related': [proj['_id']],
                                                   'type': core.PROJECT_TYPE,
                                                   'inverse_rel': 'activities',
                                                   'create_as_inverse': True},
                                       'inputs': {'related': [input['_id'] for input in [revision, source]],
                                                  'type': core.REVISION_TYPE,
                                                  'inverse_rel': 'activities'},
                                       'outputs': {'related': [output['_id'] for output in [output_revision]],
                                                  'type': core.REVISION_TYPE,
                                                  'inverse_rel': 'origins'},
                                       'actions': {'related': [r['_id'] for r in []],
                                                   'type': core.REVISION_TYPE,
                                                   'inverse_rel': 'procedures'}}}

    activity = activities.create_activity(s, proj, sentinel.activity_name,
                                          inputs=[revision, source],
                                          outputs=[output_path])

    assert_is_not_none(activity)
    upload_file.assert_called_with(s, proj, output_path)
    s.post.assert_called_with(sentinel.activities_url, data={'activities': [expected_post]})
    simplify_response.assert_called_once_with(post_activity_return)



import six
import os
import ovation.download

from tqdm import tqdm

def download_resources(session,
                       workflow,
                       activity_label,
                       resource_label, output=None, progress=tqdm):
    """
    Download the Resources for a labeled activity. Resources with the given label are downloaded
    to the given output directory.

    :param session: ovation.session.Session
    :param workflow: workflow Dict
    :param activity_label: configured activity label (unique within workflow)
    :param resource_label: desired resource_label
    :param output: output directory path
    :param progress: tqdm-like progress indicator

    """

    activity = session.get(workflow.relationships[activity_label].related)

    # Find the activity resources with the given label
    resources = [r for r in activity.activity.resources if r.label == resource_label]

    for r in resources:
        ovation.download.download_url(r.read_url, output=output, progress=progress)





import six
from ovation.session import simplify_response


FOLDER_TYPE = 'Folder'
FILE_TYPE = 'File'
REVISION_TYPE = 'Revision'
PROJECT_TYPE = 'Project'
ACTIVITY_TYPE = 'Activity'


def add_link(session, entity,
             target=None,
             rel=None,
             inverse_rel=None):
    """
    Adds a new relationship link from entity to the target. All relationships in
    Ovation are many-to-many.

    :param session: ovation.session.Session
    :param entity: relationship source entity (dict or ID)
    :param target: relationship target ID
    :param rel: relationship `rel`
    :param inverse_rel: inverse relationship (optional)
    :return: {'links': [LinkInfo],
                'entities': [Updated entities]}
    """

    if target is None:
        raise Exception("Target required")
    if rel is None:
        raise Exception("Rel required")

    entity = get_entity(session, entity)
    if (not 'relationships' in entity) or (not rel in entity['relationships']):
        raise Exception("Entity does not have a rel {}".format(rel))

    link = {'target_id': target}
    if inverse_rel:
        link['inverse_rel'] = inverse_rel

    return session.post(entity['relationships'][rel]['self'], data=[link])


def remove_link(session, entity,
                target=None,
                rel=None):

    """
    Removes relationship links from entity to the target with the given `rel`.

    :param session: ovation.session.Session
    :param entity: relationship source entity (dict or ID)
    :param target: relationship target ID
    :param rel: relationship `rel`
    """

    if target is None:
        raise Exception("Target required")
    if rel is None:
        raise Exception("Rel required")

    entity = get_entity(session, entity)
    if (not 'relationships' in entity) or (not rel in entity['relationships']):
        raise Exception("Entity does not have a rel {}".format(rel))

    relationships = session.get(entity['relationships'][rel]['self'])
    for r in relationships:
        if r['target_id'] == target:
            session.delete(session.path('relationships',
                                        entity_id=r['_id'],
                                        org=entity['organization_id']))


def create_file(session, parent, name, attributes=None):
    """
    Create a new `File` entity under parent.

    :param session: ovation.session.Session
    :param parent: parent Project or Folder (entity dict or ID)
    :param name: File name
    :param attributes: additional attributes for the file
    :return: created file
    """

    if attributes is None:
        attributes = {}

    return _create_contents(session, FILE_TYPE, parent, name, attributes=attributes)


def _create_contents(session, entity_type, parent, name, attributes=None):
    if attributes is None:
        attributes = {}

    if isinstance(parent, six.string_types):
        parent = session.get(session.path('entities', entity_id=parent))

    attributes.update({'name': name})

    attr = {'type': entity_type,
            'attributes': attributes}

    result = session.post(parent['links']['self'], data={'entities': [attr]})
    return simplify_response(result['entities'][0])


def create_folder(session, parent, name, attributes=None):
    """
    Create a new `Folder` entity under parent.

    :param session: ovation.session.Session
    :param parent: parent Project or Folder (entity dict or ID)
    :param name: Folder name
    :param attributes: additional attributes for the folder
    :return: created folder
    """

    if attributes is None:
        attributes = {}

    return _create_contents(session, FOLDER_TYPE, parent, name, attributes=attributes)


def delete_entity(session, entity):
    """
    Deletes an entity. Deleted entities are put in the "trash" and are no longer visible
    or returned by GET operations. Trashed entities can be restored from the trash by
    calling `undelete_entity`.

    :param session: ovation.session.Session
    :param entity: entity dict or ID
    :return: deleted entity Dict
    """

    try:
        id = entity['_id']
    except TypeError:
        id = entity

    return session.delete(session.path(entity_id=id))


def undelete_entity(session, entity):
    """
    Undeletes an entity

    :param session: ovation.session.Session
    :param entity: entity dict or ID
    :return: undeleted entity Dict
    """

    if isinstance(entity, six.string_types):
        entity = get_entity(session, entity, include_trash=True)

    return session.put(session.path(entity_id=entity['_id']) + "/restore", entity)


def get_entity(session, id, include_trash=False, include_annotations=False):
    """
    Gets an entity by ID

    :param session: ovation.session.Session
    :param id: entity ID
    :param: include_trash: if true, return trashed entities
    :param: include_annotations: if true, include annotations with results
    :return: entity Dict or {"entity": <entity>, "includes": [..includes..]} if include_annotations is True
    """

    if isinstance(id, six.string_types):
        params = {"trash": str(include_trash).lower()}
        if include_annotations:
            params["includes"] = "annotations"
        return session.get(session.path(entity_id=id), params=params)

    return id


def get_projects(session):
    """
    Gets all Projects visible to the authenticated user
    :param session: ovation.session.Session
    :return: list of Projects
    """
    return session.get(session.path(resource='projects'))



import six
import os

import ovation.lab.upload as upload

from tqdm import tqdm


def create_activity(session, workflow_id, activity_label, activity=None,
                    resources=None, resource_groups=None,
                    progress=tqdm):
    """
    Creates a new workflow activity.

    :param session: ovation.session.Session
    :param workflow_id: workflow ID
    :param activity_label: activity label
    :param activity: activity record
    :param resources: local path(s) to activity Resources (by label)
    :param resource_groups: local path(s) to activity ResourceGroups (by label)
    :return: newly created Activity dict
    """

    if resource_groups is None:
        resource_groups = {}
    if resources is None:
        resources = {}
    if activity is None:
        activity = {}

    if len(activity.keys()) > 0 and not 'custom_attributes' in activity:
        activity = {'custom_attributes': activity}

    workflow = session.get(session.path('workflows', workflow_id)).workflow
    activity_path = workflow.relationships[activity_label].self

    if len(resources) > 0 or len(resource_groups) > 0:
        activity['complete'] = False

    activity = session.post(activity_path, data={'activity': activity}).activity

    for (label, paths) in six.iteritems(resources):
        for local_path in paths:
            upload.upload_resource(session, activity['uuid'], local_path, label=label, progress=progress)

    for (label, paths) in six.iteritems(resource_groups):
        for local_path in paths:
            upload.upload_resource_group(session, activity, local_path, label=label, progress=progress)

    return activity


def get_activity(session, workflow, label):
    return session.get(workflow.relationships[label].related).activity


def get_workflow(session, workflow):
    return session.get(session.path('workflow'), workflow)['workflow']


def get_samples(session, workflow):
    """
    Gets all samples in a workflow.

    :param session: ovation.session.Session
    :param workflow: workflow Id or Dict
    :return: list of Sample records
    """

    if isinstance(workflow, int):
        wf = session.get(session.path('workflow', workflow))
        return wf['samples']
    else:
        return workflow['samples']



import os.path
import ovation.session
import ovation.transfer as transfer
from ovation.session import DataDict

from unittest.mock import Mock, sentinel, patch, call
from nose.tools import istest, assert_equal, with_setup


def setup():
    pass


def teardown():
    if os.path.isfile('.checkpoint.csv'):
        os.remove('.checkpoint.csv')


@istest
@patch('ovation.upload.guess_content_type')
@patch('boto3.Session')
@patch('ovation.core.create_file')
@with_setup(setup, teardown)
def should_create_file(create_file, boto_session, guess_content_type):
    create_file.return_value = ovation.session.DataDict(
            {'links': ovation.session.DataDict({'self': sentinel.file_self})})

    session = Mock(spec=ovation.session.Session)
    session.post.return_value = {'entities': [{'type': 'Revision',
                                               '_id': sentinel.revision_id,
                                               'links': {'self': sentinel.revision_self,
                                                         'upload-complete': sentinel.update_complete},
                                               'attributes': {'name': sentinel.file_name,
                                                              'content_type': sentinel.content_type}}],
                                 'aws': [{'aws': {'key': sentinel.aws_key,
                                                  'access_key_id': sentinel.access_key_id,
                                                  'secret_access_key': sentinel.secret_access_key,
                                                  'session_token': sentinel.session_token}}]}
    session.put.return_value = sentinel.updated_revision
    session.path.return_value = sentinel.put_path

    guess_content_type.return_value = sentinel.content_type

    aws_session = Mock()
    boto_session.return_value = aws_session
    s3 = Mock()
    aws_session.resource.return_value = s3
    file_obj = Mock()
    s3.Object.return_value = file_obj

    aws_response = {'VersionId': sentinel.version_id}
    file_obj.copy_from.return_value = aws_response

    assert_equal(transfer.copy_file(session,
                                    sentinel.parent_folder_id,
                                    sentinel.file_key,
                                    sentinel.file_name,
                                    sentinel.source_bucket,
                                    sentinel.dest_bucket,
                                    sentinel.global_access_key_id,
                                    sentinel.gobal_secret_access_key),
                 sentinel.updated_revision)

    session.post.assert_called_with(sentinel.file_self,
                                    data={'entities': [{'type': 'Revision',
                                                        'attributes': {'name': sentinel.file_name,
                                                                       'content_type': sentinel.content_type}}]})
    file_obj.copy_from.assert_called_with(CopySource='{}/{}'.format(sentinel.source_bucket, sentinel.file_key))
    session.put.assert_called_with(sentinel.update_complete,
                                   entity=None)


def mock_copy_file(session, parent=None, file_key=None, file_name=None, source_bucket=None,
                   destination_bucket=None, aws_access_key_id=None, aws_secret_access_key=None):
    return {'_id': sentinel.revision1}


@istest
@patch('ovation.transfer._make_session')
@patch('ovation.core.create_folder')
@patch('boto3.Session')
@with_setup(setup, teardown)
def should_copy_bucket_contents(boto_session, create_folder, make_session):
    aws_session = Mock()
    s3 = Mock()
    bucket = Mock()

    boto_session.return_value = aws_session
    aws_session.resource.return_value = s3
    s3.Bucket.return_value = bucket

    keys = [DataDict({'key': 'f1/'}),
            DataDict({'key': 'f1/f2/'}),
            DataDict({'key': 'f1/f2/f3/'}),
            DataDict({'key': 'f1/file1.txt'})]

    bucket.objects.all.return_value = keys

    create_folder.side_effect = [{'_id': sentinel.f1}, {'_id': sentinel.f2}, {'_id': sentinel.f3}]

    make_session.return_value = sentinel.ov_session

    session = DataDict({'token': sentinel.ov_session_token})

    r = transfer.copy_bucket_contents(session,
                                      project=sentinel.project,
                                      aws_access_key_id=sentinel.access_key_id,
                                      aws_secret_access_key=sentinel.secret_access_key,
                                      source_s3_bucket=sentinel.src_bucket,
                                      destination_s3_bucket=sentinel.dest_bucket,
                                      copy_file_fn=mock_copy_file)

    assert_equal(set(map(str, r.values())), set(map(str, {sentinel.revision1, sentinel.f1, sentinel.f2, sentinel.f3})))



from unittest.mock import sentinel, Mock

from nose.tools import istest, assert_equal

import ovation.download as download
import ovation.session as session


@istest
def should_get_download_info():
    revision = {'type': 'Revision',
                'attributes': {'url': sentinel.url}}

    s = Mock(spec=session.Session)
    s.session = Mock()
    response = Mock()
    response.json = Mock(return_value=sentinel.result)
    s.session.get = Mock(return_value=response)
    s.token = sentinel.token

    result = download.revision_download_info(s, revision)

    assert_equal(result, sentinel.result)
    s.session.get.assert_called_with(sentinel.url,
                                     headers={'accept': 'application/json'},
                                     params={'token': sentinel.token})



import copy
import math
import boto3
from unittest.mock import Mock, sentinel, patch, ANY

from nose.tools import istest, assert_equal

import ovation.session
import ovation.upload as upload

from boto3.s3.transfer import TransferConfig

@istest
@patch('os.path.getsize')
@patch('boto3.Session')
def should_create_revision(boto_session, getsize):
    file = {'type': 'File',
            'links': {'self': sentinel.self}}
    path = '/local/path/file.txt'
    rev = {'_id': 1,
           'type': 'Revision',
           'attributes': {'url': sentinel.url},
           'links': {'upload-complete': sentinel.upload_complete}}

    getsize.return_value = 100

    s = Mock(spec=ovation.session.Session)

    def entity_path(type='', id=''):
        return "/api/v1/{}/{}".format(type, id)

    s.path.side_effect = entity_path
    aws_session = Mock()
    s3 = Mock()
    boto_session.return_value = aws_session
    aws_session.resource = Mock(return_value=s3)

    file_obj = Mock()
    s3.Object = Mock(return_value=file_obj)
    file_obj.upload_file = Mock()
    file_obj.version_id = sentinel.version

    s.post = Mock(return_value={'entities': [rev],
                                'aws': [{'aws': dict(access_key_id=sentinel.access_key,
                                                     secret_access_key=sentinel.secret_key,
                                                     session_token=sentinel.session_token,
                                                     bucket=sentinel.bucket,
                                                     key=sentinel.key)}]})

    s.put = Mock(return_value=sentinel.result)

    # Act
    result = upload.upload_revision(s, file, path, progress=None)

    # Assert
    boto_session.assert_called_with(aws_access_key_id=sentinel.access_key,
                                    aws_secret_access_key=sentinel.secret_key,
                                    aws_session_token=sentinel.session_token)
    s3.Object.assert_called_with(sentinel.bucket, sentinel.key)
    file_obj.upload_file.assert_called_with(path,
                                            ExtraArgs={'ContentType': 'text/plain',
                                                       'ServerSideEncryption': 'AES256'},
                                            Config=ANY)

    s.put.assert_called_with(sentinel.upload_complete, entity=None)

    assert_equal(result, sentinel.result)


@istest
@patch('os.path.getsize')
@patch('boto3.Session')
def should_create_revision_from_file_obj(boto_session, getsize):
    file = {'type': 'File',
            'links': {'self': sentinel.self}}
    open_file_obj = sentinel.file_obj
    rev = {'_id': 1,
           'type': 'Revision',
           'attributes': {'url': sentinel.url},
           'links': {'upload-complete': sentinel.upload_complete}}

    getsize.return_value = 100

    s = Mock(spec=ovation.session.Session)

    def entity_path(type='', id=''):
        return "/api/v1/{}/{}".format(type, id)

    s.path.side_effect = entity_path
    aws_session = Mock()
    s3 = Mock()
    boto_session.return_value = aws_session
    aws_session.resource = Mock(return_value=s3)

    file_obj = Mock()
    s3.Object = Mock(return_value=file_obj)
    file_obj.upload_file = Mock()
    file_obj.version_id = sentinel.version

    s.post = Mock(return_value={'entities': [rev],
                                'aws': [{'aws': dict(access_key_id=sentinel.access_key,
                                                     secret_access_key=sentinel.secret_key,
                                                     session_token=sentinel.session_token,
                                                     bucket=sentinel.bucket,
                                                     key=sentinel.key)}]})

    s.put = Mock(return_value=sentinel.result)

    # Act
    result = upload.upload_revision(s,
                                    file,
                                    open_file_obj,
                                    content_type='text/plain',
                                    file_name='my-file',
                                    progress=None)

    # Assert
    boto_session.assert_called_with(aws_access_key_id=sentinel.access_key,
                                    aws_secret_access_key=sentinel.secret_key,
                                    aws_session_token=sentinel.session_token)
    s3.Object.assert_called_with(sentinel.bucket, sentinel.key)
    file_obj.upload_fileobj.assert_called_with(open_file_obj,
                                               ExtraArgs={'ContentType': 'text/plain',
                                                          'ServerSideEncryption': 'AES256'})

    s.put.assert_called_with(sentinel.upload_complete, entity=None)

    assert_equal(result, sentinel.result)

@istest
@patch('os.path.getsize')
@patch('boto3.Session')
def should_set_multipart_chunk_size(boto_session, getsize):
    file = {'type': 'File',
            'links': {'self': sentinel.self}}
    path = '/local/path/file.txt'
    rev = {'_id': 1,
           'type': 'Revision',
           'attributes': {'url': sentinel.url},
           'links': {'upload-complete': sentinel.upload_complete}}

    getsize.return_value = sentinel.file_size

    s = Mock(spec=ovation.session.Session)

    def entity_path(type='', id=''):
        return "/api/v1/{}/{}".format(type, id)

    s.path.side_effect = entity_path
    aws_session = Mock()
    s3 = Mock()
    boto_session.return_value = aws_session
    aws_session.resource = Mock(return_value=s3)

    file_obj = Mock()
    s3.Object = Mock(return_value=file_obj)
    file_obj.upload_file = Mock()
    file_obj.version_id = sentinel.version

    _chunk_size = Mock(return_value=sentinel.chunk_size)

    s.post = Mock(return_value={'entities': [rev],
                                'aws': [{'aws': dict(access_key_id=sentinel.access_key,
                                                     secret_access_key=sentinel.secret_key,
                                                     session_token=sentinel.session_token,
                                                     bucket=sentinel.bucket,
                                                     key=sentinel.key)}]})

    s.put = Mock(return_value=sentinel.result)

    # Act
    upload.upload_revision(s, file, path, chunk_size=_chunk_size, progress=None)

    # Assert
    call = file_obj.upload_file.call_args_list[0]
    assert_equal(call[1]['Config'].multipart_chunksize, sentinel.chunk_size)


@istest
def calculates_chunk_size_for_large_file():
    nbytes = 180 * upload.GB
    cs = upload.multipart_chunksize(nbytes)
    assert_equal(cs, math.ceil(nbytes/upload.MAX_PARTS))

@istest
def calculates_chunk_size_for_small_file():
    nbytes = 1000
    cs = upload.multipart_chunksize(nbytes)

    assert_equal(cs, 8 * upload.MB)

@istest
def MB_is_mega():
    assert_equal(upload.MB, boto3.s3.transfer.MB)

